@@ -17,8 +17,11 @@
 #    <http://www.gnu.org/licenses/>.
 
 
+from typing import Dict, List
+
+
 class Rule:
-    def __init__(self, klass, **kw):
+    def __init__(self, klass: str, **kw: str) -> None:
 
         self.klass = klass
         self.kw = "; ".join(
@@ -26,21 +29,29 @@ class Rule:
         ).strip()
         self.kwl = [(k.replace("_", "-"), kw[k][1:]) for k in sorted(kw.keys())]
 
-    def __str__(self):
+    def __str__(self) -> str:
         return "%s { %s; }" % (self.klass, self.kw)
 
 
-def index(r, g, b):
+def index(r: int, g: int, b: int) -> str:
+    """
+    Implements the 6x6x6 color cube location of 8bit mode described at
+    https://en.wikipedia.org/wiki/ANSI_escape_code#8-bit
+    """
     return str(16 + (r * 36) + (g * 6) + b)
 
 
-def color_component(x):
+def color_component(x: int) -> int:
+    """
+    Implements the 6x6x6 color cube values of 8bit mode described at
+    https://en.wikipedia.org/wiki/ANSI_escape_code#8-bit
+    """
     if x == 0:
         return 0
     return 0x37 + (0x28 * x)
 
 
-def color(r, g, b):
+def color(r: int, g: int, b: int) -> str:
     return "#%.2x%.2x%.2x" % (
         color_component(r),
         color_component(g),
@@ -48,11 +59,19 @@ def color(r, g, b):
     )
 
 
-def level(grey):
+def level(grey: int) -> str:
+    """
+    Implements 24 grey values of 8bit mode described at
+    https://en.wikipedia.org/wiki/ANSI_escape_code#8-bit
+    """
     return "#%.2x%.2x%.2x" % (((grey * 10) + 8,) * 3)
 
 
-def index2(grey):
+def index2(grey: int) -> str:
+    """
+    Implements 24 grey location of 8bit mode described at
+    https://en.wikipedia.org/wiki/ANSI_escape_code#8-bit
+    """
     return str(232 + grey)
 
 
@@ -95,6 +114,7 @@ SCHEME = {
         "#00ffff",
         "#ffffff",
     ),
+    # Based on the "osx" palette in previous versions of ansi2html.
     "osx": (
         "#000000",
         "#c23621",
@@ -104,8 +124,54 @@ SCHEME = {
         "#d338d3",
         "#33bbc8",
         "#cbcccd",
-    )
-    * 2,
+        "#404040",
+        "#ff7661",
+        "#65fc64",
+        "#eded67",
+        "#896eff",
+        "#ff78ff",
+        "#73fbff",
+        "#ffffff",
+    ),
+    # Based on the "Basic" palette in macOS Terminal.
+    "osx-basic": (
+        "#000000",
+        "#800000",
+        "#008000",
+        "#808000",
+        "#000080",
+        "#800080",
+        "#008080",
+        "#808080",
+        "#666666",
+        "#e60000",
+        "#00d900",
+        "#e6e600",
+        "#0000ff",
+        "#e600e6",
+        "#00e6e6",
+        "#e6e6e6",
+    ),
+    # Based on the "Solid Colors" palette in macOS Terminal.
+    # The colors are brighter than osx-basic.
+    "osx-solid-colors": (
+        "#000000",
+        "#990000",
+        "#00a600",
+        "#999900",
+        "#0000b3",
+        "#b300b3",
+        "#00a6b3",
+        "#bfbfbf",
+        "#666666",
+        "#e60000",
+        "#00d900",
+        "#e6e600",
+        "#0000ff",
+        "#e600e6",
+        "#00e6e6",
+        "#e6e6e6",
+    ),
     # http://ethanschoonover.com/solarized
     "solarized": (
         "#262626",
@@ -163,15 +229,22 @@ SCHEME = {
     ),
 }
 
+# to be filled in runtime, when truecolor found
+truecolor_rules: List[Rule] = []
 
-def intensify(color, dark_bg, amount=64):
+
+def intensify(color: str, dark_bg: bool, amount: int = 64) -> str:
     if not dark_bg:
         amount = -amount
     rgb = tuple(max(0, min(255, amount + int(color[i : i + 2], 16))) for i in (1, 3, 5))
     return "#%.2x%.2x%.2x" % rgb
 
 
-def get_styles(dark_bg=True, line_wrap=True, scheme="ansi2html"):
+def get_styles(
+    dark_bg: bool = True,
+    line_wrap: bool = True,
+    scheme: str = "ansi2html",
+) -> List[Rule]:
     css = [
         Rule(
             ".ansi2html-content",
@@ -181,13 +254,10 @@ def get_styles(dark_bg=True, line_wrap=True, scheme="ansi2html"):
         ),
         Rule(".body_foreground", color=("#000000", "#AAAAAA")[dark_bg]),
         Rule(".body_background", background_color=("#AAAAAA", "#000000")[dark_bg]),
-        Rule(
-            ".body_foreground > .bold,.bold > .body_foreground, body.body_foreground > pre > .bold",
-            color=("#000000", "#FFFFFF")[dark_bg],
-            font_weight=("bold", "normal")[dark_bg],
-        ),
-        Rule(".inv_foreground", color=("#000000", "#FFFFFF")[not dark_bg]),
+        Rule(".inv_foreground", color=("#000000", "#AAAAAA")[not dark_bg]),
         Rule(".inv_background", background_color=("#AAAAAA", "#000000")[not dark_bg]),
+        # These effects are "SGR (Select Graphic Rendition) parameters"
+        # https://en.wikipedia.org/wiki/ANSI_escape_code#SGR_(Select_Graphic_Rendition)_parameters
         Rule(".ansi1", font_weight="bold"),
         Rule(".ansi2", font_weight="lighter"),
         Rule(".ansi3", font_style="italic"),
@@ -198,27 +268,40 @@ def get_styles(dark_bg=True, line_wrap=True, scheme="ansi2html"):
         Rule(".ansi9", text_decoration="line-through"),
     ]
 
-    # set palette
-    pal = SCHEME[scheme]
+    try:
+        pal = SCHEME[scheme]
+    except KeyError as e:
+        raise ValueError(f"Unsupported color scheme {scheme!r}") from e
+
+    if len(pal) < 16:
+        raise Exception(
+            f"Color scheme {scheme!r} specifies fewer than 16 colors. 16 colors are required."
+        )
+
+    # This is 8x2 palette of 3/4-bit color mode described at
+    # https://en.wikipedia.org/wiki/ANSI_escape_code#3-bit_and_4-bit
+    # .ansi{30..37} is foreground
+    # .ansi{40..47} is background
     for _index in range(8):
         css.append(Rule(".ansi3%s" % _index, color=pal[_index]))
         css.append(Rule(".inv3%s" % _index, background_color=pal[_index]))
     for _index in range(8):
         css.append(Rule(".ansi4%s" % _index, background_color=pal[_index]))
         css.append(Rule(".inv4%s" % _index, color=pal[_index]))
-    for _index in range(8):
-        css.append(Rule(".ansi9%s" % _index, color=intensify(pal[_index], dark_bg)))
-        css.append(
-            Rule(".inv9%s" % _index, background_color=intensify(pal[_index], dark_bg))
-        )
-    for _index in range(8):
-        css.append(
-            Rule(".ansi10%s" % _index, background_color=intensify(pal[_index], dark_bg))
-        )
-        css.append(Rule(".inv10%s" % _index, color=intensify(pal[_index], dark_bg)))
 
-    # set palette colors in 256 color encoding
-    pal = SCHEME[scheme]
+    # This is the 8x2 bright(!) palette of 4-bit color mode described at
+    # https://en.wikipedia.org/wiki/ANSI_escape_code#3-bit_and_4-bit
+    # .ansi{90..97} is foreground
+    # .ansi{100..107} is background
+    for _index in range(8):
+        css.append(Rule(".ansi9%s" % _index, color=pal[_index + 8]))
+        css.append(Rule(".inv9%s" % _index, background_color=pal[_index + 8]))
+    for _index in range(8):
+        css.append(Rule(".ansi10%s" % _index, background_color=pal[_index + 8]))
+        css.append(Rule(".inv10%s" % _index, color=pal[_index + 8]))
+
+    # This is the first 16 palette slots of 8-bit color mode described at
+    # https://en.wikipedia.org/wiki/ANSI_escape_code#8-bit
     for _index in range(len(pal)):
         css.append(Rule(".ansi38-%s" % _index, color=pal[_index]))
         css.append(Rule(".inv38-%s" % _index, background_color=pal[_index]))
@@ -228,6 +311,10 @@ def get_styles(dark_bg=True, line_wrap=True, scheme="ansi2html"):
 
     # css.append("/* Define the explicit color codes (obnoxious) */\n\n")
 
+    # This is the 6x6x6 color cube of 8-bit mode described at
+    # https://en.wikipedia.org/wiki/ANSI_escape_code#8-bit
+    # .ansi38-{16..231} is foreground
+    # .ansi48-{16..231} is background
     for green in range(0, 6):
         for red in range(0, 6):
             for blue in range(0, 6):
@@ -256,10 +343,36 @@ def get_styles(dark_bg=True, line_wrap=True, scheme="ansi2html"):
                     )
                 )
 
+    # This is the 24 shades of grey of 8-bit mode described at
+    # https://en.wikipedia.org/wiki/ANSI_escape_code#8-bit
+    # .ansi38-{232..255} is foreground
+    # .ansi48-{232..255} is background
     for grey in range(0, 24):
         css.append(Rule(".ansi38-%s" % index2(grey), color=level(grey)))
         css.append(Rule(".inv38-%s" % index2(grey), background=level(grey)))
         css.append(Rule(".ansi48-%s" % index2(grey), background=level(grey)))
         css.append(Rule(".inv48-%s" % index2(grey), color=level(grey)))
 
+    css.extend(truecolor_rules)
+
     return css
+
+
+# as truecolor encoding has 16 millions colors, adding only used colors during parsing
+def add_truecolor_style_rule(
+    is_foreground: bool, ansi_code: int, r: int, g: int, b: int, parameter: str
+) -> None:
+    rule_name = ".ansi{}-{}".format(ansi_code, parameter)
+    color = "#{:02X}{:02X}{:02X}".format(r, g, b)
+    if is_foreground:
+        rule = Rule(rule_name, color=color)
+    else:
+        rule = Rule(rule_name, background_color=color)
+    truecolor_rules.append(rule)
+
+
+def pop_truecolor_styles() -> Dict[str, Rule]:
+    global truecolor_rules  # pylint: disable=global-statement
+    styles = dict([(item.klass.strip("."), item) for item in truecolor_rules])
+    truecolor_rules = []
+    return styles
